;ComboLok.asm - New button push routine -- "BUTTON."

;Updated: December 30, 2010
;  
;5 bytes (64 bits) reserved for opening code.  
;EEPROM contains permanent code which must be programmed before use
;  (first byte contains number of bits programmed)

;Dit-Dah entries used for code to open lock.

;HoldDown count of 13 goes into programming mode to set up new entry code.
;   Dit-Dah entries from one up to 32 are permitted.

;Internal oscillator in ATTINY2313 is running micro-controller at 1MHz

;Sleep mode added - 1/25/2006

;Project Original: January 25, 2006

;----------------------------------------------------

;       HOLDDOWN, LED FLASH, COMMANDS:

;13. Enter new code using dit-dah entries. One to 32 dit-dah entries possible.
;    (this is entered into EEPROM for permanent storage after a power down)

;r0-r4 = SRAM TP code entry registers (r0=No. of bits, r1-4 = 4 bytes for code)
;r5-r9 = SRAM duplicate code copied from EEPROM (r5= No. of bits, r6-9 = 4 bytes)

;-------------------------------------------------------


.include "tn2313def.inc"

rjmp	START1
rjmp	Decode

	
START1:				;set up stack pointer and initialize cold start

ldi		r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out		SPL,R16				; Initialize stack pointer to end of internal RAM

ldi		r16,7		;Set bits 0-3 for output
out		DDRA,r16	;Execute data direction with PA0-3 as output

clr		r16
out		PortA,r16	;turn off ports

ldi		r16,122		;Set bits PD1,3,4,5,6 to ones for out, bit PD0,2 for input
out		DDRD,r16	;Execute data direction with PD0 as input rest as output
					;Two inputs on this design because of wiring change accomodation
					;  Use only PD2 as input on future wirings

clr		r16
out		PortD,r16	;LED and relay coil is off when port pin is low
					;  This also disables pullup resistor for bit 0

ser		R16			;Set all bits for data direction as outputs
out		DDRB,R16	;execute data direction with all as outputs
clr		r16
out		PORTB,R16	;execute to set out ports low

in		r16,GIMSK	;enable INT0 (D port, bit 2(4))
ori		r16,64		; (bit 6 = INT0)
out		GIMSK,r16

clr		r18			;clear flags register R18

rcall	FetchEEPROM	;grab last recorded code data and put in R5-r9

bset	7			;enable global interrupts


;-----------------------------

MainLoop:		;Awaits interrupt request by touch of touch plate

ldi		r16,10


MainLoop1:

sbiw	r30,1
brne	MainLoop1
dec		r16
brne	MainLoop1

sbi		PortB,0		;ON program LED - blinks LED every 3.5 seconds
rcall	Delay40th
cbi		PortB,0		;Off program LED

rjmp	MainLoop


;-----------------------------

Decode:		;Interrupt Routine: Decodes touches from touch plate


push	r16			;save value in r16

in		r16,SREG
sts		$60,r16		;Save SREG contents in RAM

in		r16,MCUCR	;Clear SE bit 5(32) to prevent accidental sleep
andi	r16,223		
out		MCUCR,r16

rcall	Button

cpi		r20,13		
brne	ML2			;branch if it is NOT a holddown count of 13

rcall	FlashFour	;flash LED four times


ML1a:

sbic	PIND,2		;input touch plate
rjmp	ML1a

rcall	Button

rcall	WriteEEPROM	;write data entered by Button Push into EEPROM
mov		r5,r0
mov		r6,r1
mov		r7,r2
mov		r8,r3
mov		r9,r4		;move data code into SRAM
ldi		r16,12
rcall	VariableFlashBoth


ML1b:

rcall	FlashFour	;flash LEDs four times on exit


DecodeEnd:

lds		r16,$60			;retrieve SREG
out		SREG,r16
pop		r16				;restore r16 value

reti					;return from interrupt


ML2:

rcall	CheckMatch
sbrc	r18,0		;skip if r18, bit 0(1) is clear
rjmp	ML2a

rjmp	ML1b		;flash LED and exit through reti


ML2a:

sbi		PortB,1		;turn on red OUTSIDE LED
sbi		PortB,0		;turn on red PROGRAM LED


ML4:

sbic	PIND,2		;check for switch down - skip if switch down
rjmp	ML4

rcall	Button
cpi		r20,0
breq	ML5			;Off LED and branch to MainLoop

cbi		PortB,0			;turn red PROGRAM LED off
cbi		PortB,1			;turn red OUTSIDE LED off
sbi		PortB,7			;turn on relay for pulse
rcall	DelayThreeFourths	;3/4 second delay
cbi		PortB,7			;turn off relay

rjmp	DecodeEnd


ML5:

cbi		PortB,0		;off PROGRAM LED
cbi		PortB,1		;off OUTSIDE LED
rjmp	DecodeEnd


;------------------------

Delay40th:			; 1/40th second delay

push	r31
push	r0
push	r1

ldi		r31,33		; high byte
mov		r1,r31		; set up r1 for 1/40th second delay
clr		r0
ldi		r31,1		;no delay for r31


DelayA:

dec		r0
brne	DelayA

dec		r1
brne	DelayA

dec		r31
brne	DelayA

pop		r1
pop		r0
pop		r31

ret

;----------------------------------

DelayThreeFourths:	;3/4 second delay

push	r16
ldi		r16,30
rjmp	Delay6a


;----------------------------------


DelayOne:		;1 second delay

push	r16
ldi		r16,40
rjmp	Delay6a

;------------------------------------


Delay6:				;6 second delay

push	r16
clr		r16


Delay6a:

rcall	Delay40th
dec		r16
brne	Delay6a

pop		r16
ret

;---------------------------------------


Delay15:			;15 second delay routine

push 	r0
push	r16
clr		r0
ldi		r16,3
mov		r0,r16
clr		r16


Delay15a:

rcall	Delay40th	;1/40th second delay
dec		r16
brne	Delay15a

dec		r0
brne	Delay15a

pop		r16
pop		r0
ret


;------------------------------------

FlashFour:			;rapid four flashes of Outside and Program LEDs

ldi		r16,4
rjmp	VF1			


;-----------------------------------

VariableFlashBoth:

VF1:

sbi		PortB,1		;on OUTSIDE LED
rcall	Delay40th
cbi		PortB,1		;off OUTSIDE LED
sbi		PortB,0		;on PROGRAM LED
rcall	Delay40th
cbi		PortB,0		;off PROGRAM LED
rcall	Delay40th

dec		r16
brne	VF1

ret


;---------------------------------------

Button:		;Replaces old ButtonPush.  1/11/2006

push	r23

clr		r20		;HoldDown count
clr		r0		;clear number of bits in code
clr		r1		;clear lowest code count
clr		r2		;clear next code count
clr		r3		;clear next code count
clr		r4		;clear next code count (clear all code counts)

clr		r23


B1:

inc		r23
rcall	Delay40th
cpi		r23,120		;delay before first holddown count - 40=one second - 3 seconds
brsh	B3			;branch if - it is a holddown count

sbis	PIND,2		;skip if switch is NOT down
rjmp	B1


B2:

cpi		r23,20		;compare r23 for 20 (1/2 second dah count)
brlo	B2b			;branch if lower (dit count)


B2a:				;it is dah count

sec					;set carry flag -- it is dah entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortB,0
sbi		PortB,1		;turn on LEDs both Program and Outside
rcall	Delay40th
rcall	Delay40th	;delay for longer on of LEDs
rcall	Delay40th
rcall	Delay40th


B2c:

rcall	Delay40th
cbi		PortB,0
cbi		PortB,1		;off both LEDS - Program and Outside

inc		r0			;increment bit count
clr		r23			;clear r23


B2d:

sbis	PIND,2		;skip if switch is up
rjmp	B2e			;jump back - there is a new entry coming in

inc		r23
rcall	Delay40th
cpi		r23,120		;has switch been up 120 counts? 40=one second (3 seconds)
brsh	B5

rjmp	B2d


B2e:

clr		r23
rjmp	B1


B2b:				;Dit count detected

clc					;clear carry flag -- it is dit entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortB,0
sbi		PortB,1		;on both Program and Outside LEDs

rjmp	B2c


B3:					;Holddown count has been detected

inc		r20			;increment holddown count

cpi		r20,21		;Make error flashout if touch plate left touched too long, or touch plate
					;  gets grounded out due to rain.  A count of 20 or more will stall.  Max Cnt = 20

brlo	B3a
rjmp	ErrorStall


B3a:

sbi		PortB,0
rcall	Delay40th
cbi		PortB,0		;flash PROGRAM LED
cbi		PortB,1		;make sure outside LED light goes out

clr		r23			;reset r23 to zero count


B4:

inc		r23
rcall	Delay40th
sbic	PIND,2		;skip if switch is down
rjmp	B4a

cpi		r23,30
brlo	B4

rjmp	B3

;---

ErrorStall:	;To stall if touch plate grounded by rain

sbi		PortB,0		;ON program LED
rcall	Delay40th
cbi		PortB,0		;OFF program LED
rcall	Delay40th
rcall	Delay40th
rcall	Delay40th	;Steadily flash Program LED if in stalled condition

sbis	PIND,2		;skip if touch plate, PIND,2, not touched
rjmp	ErrorStall	;loop back if touch plate is still grounded or touched

ldi		r20,0		;zero r20, hold down count
rjmp	B5			;jump out of stall and exit this routine


;----

B4a:

rcall	Delay40th


B5:

pop		r23

ret


;---------------------------------------

CheckMatch:			;Check for match between r0-4 and r5-r9

cp		r0,r5		;compare number of digits present
brne	CMOUT1

cp		r1,r6
brne	CMOUT1

cp		r2,r7
brne	CMOUT1

cp		r3,r8
brne	CMOUT1

cp		r4,r9
brne	CMOUT1

sbr		r18,1		;set r18,0(1) - there is a match
ret	


CMOUT1:

cbr		r18,1		;clear r18,0(1) - no match
ret


;-----------------------------------

FetchEEPROM:		;grabs data in permanent saved EEPROM

ldi		r17,0		;start of EEPROM memory location

FEE:

sbic	EECR,EEWE	;Wait for completion of previous write
rjmp	FEE

out		EEAR,r17	;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r5,EEDR		;read data from data register

inc		r17			;increment to next EEPROM address

out		EEAR,r17	;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r6,EEDR		;read data from data register

inc		r17			;increment to next EEPROM address

out		EEAR,r17	;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r7,EEDR		;read data from data register

inc		r17			;increment to next EEPROM address

out		EEAR,r17	;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r8,EEDR		;read data from data register

inc		r17			;increment to next EEPROM address

out		EEAR,r17	;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r9,EEDR		;read data from data register

ret					;code data now in r5-9


;-------------------------------------------------

WriteEEPROM:		;write new code from r0-r4 into EEPROM

ldi		r17,0		;start of EEPROM memory section

WEE:

sbic	EECR,EEPE
rjmp	WEE

out		EEAR,r17	;set up address in address register
out		EEDR,r0		;put contents of r0 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

inc		r17


WEE1:

sbic	EECR,EEPE
rjmp	WEE1

out		EEAR,r17	;set up address in address register
out		EEDR,r1		;put contents of r1 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

inc		r17


WEE2:

sbic	EECR,EEPE
rjmp	WEE2

out		EEAR,r17	;set up address in address register
out		EEDR,r2		;put contents of r2 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

inc		r17


WEE3:

sbic	EECR,EEPE
rjmp	WEE3

out		EEAR,r17	;set up address in address register
out		EEDR,r3		;put contents of r3 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

inc		r17


WEE4:

sbic	EECR,EEPE
rjmp	WEE4

out		EEAR,r17	;set up address in address register
out		EEDR,r4		;put contents of r4 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

ret


;--------------------------------------------------

TEST1:

rcall	DelayOne
rcall	DelayOne
tst		r26
breq	T1b

T1a:

sbi		PortB,3
rcall	Delay40th
cbi		PortB,3

rcall	DelayThreeFourths
dec		r26
brne	T1a

rcall	DelayOne
ret


T1b:

sbi		PortB,3
rcall	Delay6
cbi		PortB,3
ret


;----------------------------------------
